@@ -6,6 +6,7 @@
 
 from __future__ import print_function
 
+import                    gc
 import                    os
 import os.path         as op
 import itertools       as it
@@ -20,13 +21,16 @@ import                    random
 import                    shutil
 import                    pickle
 import                    hashlib
+import                    pathlib
 import                    textwrap
 import                    tempfile
 import                    contextlib
 
+
 import numpy as np
 from io import BytesIO
 import pytest
+import weakref
 
 import indexed_gzip as igzip
 
@@ -163,6 +167,22 @@ def test_open_mode():
             with pytest.raises(ValueError):
                 igzip.IndexedGzipFile(fileobj=fileobj).read()
 
+
+#pauldmccarthy/indexed_gzip#176
+@pytest.mark.parametrize('drop', [False, True])
+def test_open_path(testfile, nelems, seed, drop):
+
+    testfile = pathlib.Path(testfile)
+
+    with igzip._IndexedGzipFile(filename=testfile, drop_handles=drop) as f:
+
+        element = np.random.randint(0, nelems, 1)[0]
+        readval = read_element(f, element)
+
+    assert readval == element
+    assert f.closed
+
+
 @pytest.mark.parametrize('drop', [False, True])
 def test_atts(testfile, drop):
 
@@ -508,6 +528,129 @@ def test_read_beyond_end(concat, drop):
         os.remove(testfile)
 
 
+def test_exception_preserved(testfile, nelems):
+    """When wrapping a python file object, if it raises an exception
+    it should be preserved.
+    """
+    if sys.version_info[0] < 3:
+        # We can't set the .read attribute in Python 2
+        # because it's read-only, so skip it.
+        return
+
+    # We'll use a weakref to check that we are handling
+    # reference counting correctly, and you cannot
+    # weakref an Exception, so we need a subclass.
+    class MyException(Exception):
+        pass
+
+    # Function which raises an exception. This is
+    # is patched into the python file-like to test
+    # that the igzip object preserves the exception.
+    # We store a weakref to each exception to check
+    # that no references to them are kept.
+    all_errors = []
+    error_msg  = "This error should be preserved"
+
+    def error_fn(name, *args, **kwargs):
+        err = MyException(error_msg + ": " + name)
+        all_errors.append(weakref.ref(err))
+        raise err
+
+    # Below we check that each of these scenarios
+    # raises an appropriate exception
+    def test_create(pyf):
+        pyf.seek = ft.partial(error_fn, "test_create")
+        gzf = igzip.IndexedGzipFile(fileobj=pyf)
+        return gzf
+
+    def test_build_full_index(pyf):
+        pyf.read = ft.partial(error_fn, "test_build_full_index")
+        pyf.seek = ft.partial(error_fn, "test_build_full_index")
+        gzf = igzip.IndexedGzipFile(fileobj=pyf)
+        gzf.build_full_index()
+        return gzf
+
+    def test_seek(pyf):
+        pyf.seek = ft.partial(error_fn, "test_seek")
+        gzf = igzip.IndexedGzipFile(fileobj=pyf)
+        gzf.seek(5)
+        return gzf
+
+    def test_read(pyf):
+        # Test _IndexedGzipFile.read, as the buffered
+        # IndexedGzipFile.read method uses readinto
+        pyf.read = ft.partial(error_fn, "test_read")
+        gzf = igzip._IndexedGzipFile(fileobj=pyf)
+        gzf.read(1)
+        return gzf
+
+    def test_readinto(pyf):
+        pyf.read = ft.partial(error_fn, "test_readinto")
+        gzf = igzip.IndexedGzipFile(fileobj=pyf)
+        ba = bytearray(1)
+        gzf.readinto(ba)
+        return gzf
+
+    def test_export_index(pyf):
+        gzf = igzip.IndexedGzipFile(fileobj=pyf)
+        gzf.build_full_index()
+        idxf = BytesIO()
+        idxf.write = ft.partial(error_fn, "test_export_index")
+        gzf.export_index(fileobj=idxf)
+        return gzf
+
+    def test_import_index(pyf):
+        gzf = igzip.IndexedGzipFile(fileobj=pyf)
+        idxf = BytesIO()
+        gzf.build_full_index()
+        gzf.export_index(fileobj=idxf)
+        gzf.close()
+        del gzf
+        gzf = igzip.IndexedGzipFile(fileobj=pyf)
+        idxf.read = ft.partial(error_fn, "test_import_index")
+        gzf.import_index(fileobj=idxf)
+        return gzf
+
+    tests = [test_create, test_build_full_index, test_seek,
+             test_read, test_readinto, test_export_index,
+             test_import_index]
+
+    # Call the given function, making sure that
+    # the exception originating from the python
+    # file-like is preserved
+    def check_error_preserved(func):
+        pyf = None
+        gzf = None
+        with open(testfile, 'rb') as f:
+            pyf = BytesIO(f.read())
+        try:
+            gzf = func(pyf)
+        except Exception as e:
+            fname = func.__name__
+            estr = str(e) + " " + str(e.__cause__)
+            assert error_msg in estr, \
+                "Exception was not preserved; got {}".format(estr)
+            assert fname in estr, \
+                "Exception didn't come from {}; got {}".format(fname, estr)
+            del e
+        else:
+            assert False, "Expected an exception to be raised"
+
+        if gzf is not None: gzf.close()
+        if pyf is not None: pyf.close()
+        del gzf
+        del pyf
+
+    # Run the tests
+    for test in tests:
+        check_error_preserved(test)
+
+    # Make sure there are no dangling
+    # references to any exception objects
+    gc.collect()
+    for e in all_errors:
+        assert (e() is None), "Exception was not garbage collected"
+
 def test_seek(concat):
     with tempdir() as tdir:
         nelems   = 262144 # == 2MB
