@@ -132,8 +132,11 @@ if (ctest_zran is not None) and (not sys.platform.startswith("win")):
     def test_export_import_format_v0():
         ctest_zran.test_export_import_format_v0()
 
-    def test_crc_validation(concat):
-        ctest_zran.test_crc_validation(concat)
+    def test_crc_validation(concat, seed):
+        ctest_zran.test_crc_validation(concat, seed)
+
+    def test_skip_crc_with_footer_that_looks_like_new_stream(seed):
+        ctest_zran.test_skip_crc_with_footer_that_looks_like_new_stream(seed)
 
     def test_standard_usage_with_null_padding(concat):
         ctest_zran.test_standard_usage_with_null_padding(concat)
